;==============================================================================
; (c) Copyright Elect Software International Inc., 1992, Toronto. Anyone can
; use this code for anything as long as it is not resold as a software
; development resource, as long as the copyright notice isn't removed, as
; long as changes are clearly marked as to authorship, and as long as users
; indemnify Elect from any liability.
; Comments welcome. Henrik Bechmann, CIS:72701,3717; Tel:416-534-8176.
;==============================================================================

;==============================================================================
;                                BOX INTERFACE
;==============================================================================
; Description:
;     The basic facilities for tracking and displaying Boxes. These facilities
;     are used for a wide variety of popups, but Box has been presented here in
;     a simplified way to demonstrate how it is used.
;Construction...
;    Box.Constructor() ; resets box type counts, border type, box stack,
;               ; and standard colours, must be paired with Destructor()
;    Box.Destructor() ; must be paired with Constructor()
;Properties...
;   Box.VirtualDrive             ; default "", for stacking boxes
;   Box.PaintShadows             ; default true if not Monitor() = "Mono"
;   Box.MenuBoxNumber            ; initialized to 0
;   Box.MenuBoxIsAlive           ; initialized to false
;
;   Box.TopLeftChar              ; default ""
;   Box.HorizontalChar           ; default ""
;   Box.TopRightChar             ; default ""
;   Box.VerticalChar             ; default ""
;   Box.LeftCentreChar           ; default ""
;   Box.RightCentreChar          ; default ""
;   Box.BottomLeftChar           ; default ""
;   Box.BottomRightChar          ; default ""
;
;    Box.SetBorderType(VerticalSOrD,HorizontalSOrD) ; Single or Double lines
;(Usage...)
;    Box.RegisterType(BoxType) ; returns new registration number
;    Box.DeregisterType(BoxType) ; Returns previous registration number
;    Box.Display(BoxRow,BoxCol,InnerLength,InnerWidth,
;                Title,FootNote,TitleColors,
;                BodyColors,BorderColors,ShadowColors,HighLightColors)
;    ; whole rows...
;    Box.WriteRow(WriteRow,String) ; trims to window width
;    Box.ClearRow(WriteRow)
;    Box.HighLightRow(HighLightRow)
;    Box.BodyColorRow(ColorRow)
;    Box.Divider(DividerRow) ; leaves a permanent divider, border Color
;    ; partial rows...
;    Box.WriteString(WriteRow,WriteCol,String) ; trims to window width
;    Box.WriteBorderColorString(WriteRow,WriteCol,String) ; does not trim
;    Box.WriteHighlightString(WriteRow,WriteCol,String) ; does not trim
;    Box.WriteVariable(WroteRow,WriteCol,Variable) ; does not trim
;    Box.HighLightHotSpot(HighLightRow,LeftCol,RightCol,HotBoxColors)
;    ; paint area Colors...
;    Box.HighLightArea(TopLeftRow,TopLeftCol,
;                      BottomRightRow,BottomRightCol)
;    Box.BorderColorArea(TopLeftRow,TopLeftCol,
;                      BottomRightRow,BottomRightCol)
;    Box.BodyColorArea(TopLeftRow,TopLeftCol,
;                      BottomRightRow,BottomRightCol)
;    ; borders...
;    Box.WriteTitle(Title) ; trims
;    Box.WriteFootNote(FootNote) ; trims
;    Bos.WriteBlinkingFootnote(Footnote) ; trims
;    Box.MoreAboveArrow(LogicalValue)
;    Box.MoreBelowArrow(LogicalValue)
;    Box.MoreRightArrow(LogicalValue)
;    Box.MoreLeftArrow(LogicalValue)
;    ; miscellaneous...
;    Box.SetCursorPos(WindowRow,WindowCol)
;    Box.ClearMessage() ; clears paradox message by paintcanvas at box
;               ; 0,0,0,0 co-ordinates
;
;==============================================================================
;                             BOX IMPLEMENTATION
;==============================================================================
Proc Box.Constructor()
   Box.MenuBoxNumber = 0
   Box.MenuBoxIsAlive = False
   ; Other types of boxes...
   ; ...
   ; Set border style default...
   Box.SetBorderType("D","S")
   If IsFile("BX$*.*") Then
      Run norefresh norestore "del BX$*.*"
   Endif
   Box!SetChildDefaults()
   Box.VirtualDrive = ""
   If Monitor() = "Mono" Then
      Box.PaintShadows = False
   Else
      Box.PaintShadows = True
   Endif
EndProc
Writelib    LibrarySpec    Box.Constructor
Release     Procs          Box.Constructor

Proc Box!SetChildDefaults()
   Box.MenuBoxKeyAccept    = 13 ; Enter
   Box.MenuBoxKeyTop       = -71 ; Home
   Box.MenuBoxKeyBottom    = -79 ; End
   Box.MenuBoxFootnote     = "[Enter] to select, or [Esc]"

   If Monitor() <> "Mono" Then
      Box.MenuBoxTitleColors     = 48 + 14 ; Color$bCyan + Color$Yellow
      Box.MenuBoxBodyColors      = 48 + 15 ; Color$bCyan + Color$White
      Box.MenuBoxBorderColors    = 48 + 4  ; Color$bCyan + Color$Red
      Box.MenuBoxShadowColors    = 0  + 7  ; Color$bBlack + Color$LightGray
      Box.MenuBoxHighLightColors = 16 + 15 ; Color$bBlue + Color$White
   Else
      Box.MenuBoxTitleColors     = 10 ; Color$mHighText
      Box.MenuBoxBodyColors      = 10 ; Color$mHighText
      Box.MenuBoxBorderColors    = 2  ; Color$mNormal
      Box.MenuBoxShadowColors    = 0  ; Color$mNoDisplay
      Box.MenuBoxHighLightColors = 9  ; Color$mHighUnderline
   Endif
   ; Set defaults for other kinds of boxes...
EndProc
Writelib    LibrarySpec    Box!SetChildDefaults
Release     Procs          Box!SetChildDefaults

Proc Box.Destructor()
   Release Vars
      Box.MenuBoxNumber,
      Box.MenuBoxIsAlive,

      Box_WindowCol,
      Box_WindowRow,
      Box_WindowLength,
      Box_WindowWidth,
      Box_TitleColors,
      Box_WindowColors,
      Box_BorderColors,
      Box_HighLightColors,

      Box.TopLeftChar,
      Box.TopRightChar,
      Box.BottomLeftChar,
      Box.BottomRightChar,
      Box.HorizontalChar,
      Box.VerticalChar,
      Box.LeftCentreChar,
      Box.RightCentreChar,
      Box.VirtualDrive,
      Box.PaintShadows
      Box!DestroyChildDefaults()

   If IsFile("BX$*.*") Then
      Run norefresh norestore "del BX$*.*"
   Endif
EndProc
Writelib    LibrarySpec    Box.Destructor
Release     Procs          Box.Destructor

Proc Box!DestroyChildDefaults()
   Release vars

      Box.MenuBoxTitleColors,
      Box.MenuBoxBodyColors,
      Box.MenuBoxBorderColors,
      Box.MenuBoxShadowColors,
      Box.MenuBoxHighLightColors,

      Box.MenuBoxKeyAccept,
      Box.MenuBoxKeyTop,
      Box.MenuBoxKeyBottom,
      Box.MenuBoxFootnote

EndProc
Writelib    LibrarySpec    Box!DestroyChildDefaults
Release     Procs          Box!DestroyChildDefaults

Proc Box.SetBorderType(VerticalSOrD,HorizontalSOrD)
   Private
      BorderType

   BorderType = VerticalSOrD + HorizontalSOrD

   Switch
      Case BorderType = "SS":       ; Single vertical, single horizontal
         Box.TopLeftChar       = ""
         Box.HorizontalChar    = ""
         Box.TopRightChar      = ""
         Box.VerticalChar      = ""
         Box.LeftCentreChar    = ""
         Box.RightCentreChar   = ""
         Box.BottomLeftChar    = ""
         Box.BottomRightChar   = ""

      Case BorderType = "DD":       ; Double vertical, double horizontal
         Box.TopLeftChar       = ""
         Box.HorizontalChar    = ""
         Box.TopRightChar      = ""
         Box.VerticalChar      = ""
         Box.LeftCentreChar    = ""
         Box.RightCentreChar   = ""
         Box.BottomLeftChar    = ""
         Box.BottomRightChar   = ""

      Case BorderType = "SD":       ; Single vertical, double horizontal
         Box.TopLeftChar       = ""
         Box.HorizontalChar    = ""
         Box.TopRightChar      = ""
         Box.VerticalChar      = ""
         Box.LeftCentreChar    = ""
         Box.RightCentreChar   = ""
         Box.BottomLeftChar    = ""
         Box.BottomRightChar   = ""

      Case BorderType = "DS":       ; Double vertical, Single horizontal
         Box.TopLeftChar       = ""
         Box.HorizontalChar    = ""
         Box.TopRightChar      = ""
         Box.VerticalChar      = ""
         Box.LeftCentreChar    = ""
         Box.RightCentreChar   = ""
         Box.BottomLeftChar    = ""
         Box.BottomRightChar   = ""

   Endswitch
   Return 1 ; placeholder for registration number
EndProc
Writelib    LibrarySpec    Box.SetBorderType
Release     Procs          Box.SetBorderType

; Assumes that the dimensions will be in screen bounds (25 rows,80 columns)
Proc Box.Display(BoxRow,BoxCol,InnerLength,InnerWidth,
                  Title,FootNote,TitleColors,
                  BodyColors,BorderColors,ShadowColors,HighLightColors)
   Private
      ;BoxRow,BoxCol,          ; Top Left Position of the box border
      ;InnerLength,InnerWidth, ; Length and width, inside borders
                               ; including Title
      ;Title,                  ; Title line
      ;FootNote,               ; FootNote Line
      ; ..Colors - foreground and background attributes combined
      ;BodyColors,            ; Body Attribute
      ;BorderColors,          ; Border Attribute
      ;ShadowColors,          ; DropShadow Attribute
      WindowOffset,
      Variance,
      i
   Canvas Off

   ; Paint shadow...
   If Box.PaintShadows Then
      PaintCanvas
         Attribute ShadowColors
         BoxRow + 1,BoxCol +1 ,
         BoxRow + InnerLength + 2,BoxCol + InnerWidth + 4
   Endif
   ; Clear and paint area border color...
   PaintCanvas Fill " "
      Attribute BorderColors
      BoxRow,BoxCol,
      BoxRow + InnerLength + 1,BoxCol + InnerWidth + 3
   ; draw box...
   PaintCanvas Border Fill
      ; top row...
      Box.TopLeftChar +
         Fill(Box.HorizontalChar,InnerWidth) +
         Box.TopRightChar +
      ; sides...
      Fill(Box.VerticalChar, 2 * InnerLength) +
      ; bottom row...
      Box.BottomLeftChar +
         Fill(Box.HorizontalChar,InnerWidth) +
         Box.BottomRightChar
      Background
      BoxRow,BoxCol + 1,
      BoxRow + InnerLength + 1,BoxCol + InnerWidth + 2
   ; Title...
   If (Title <> "") Then
      ; draw the divider...
      PaintCanvas Fill
         Box.LeftCentreChar +
         Fill(Box.HorizontalChar,InnerWidth) +
         Box.RightCentreChar
         Background
         BoxRow + 2,BoxCol + 1,
         BoxRow + 2,BoxCol + InnerWidth + 2
      ; paint the title...
      PaintCanvas Fill
         Format("AC,W" + Strval(InnerWidth),Title)
         Attribute TitleColors
         BoxRow + 1,BoxCol + 2,
         BoxRow + 1,BoxCol + InnerWidth + 1
      WindowOffset = 2
   Else
      WindowOffset = 0
   Endif
   ; Footnote...
   If FootNote <> "" Then
      Style Attribute BorderColors
      Variance = (Len(Footnote) + 2) - InnerWidth
      If Variance > 0 Then
         FootNote = Substr(Footnote,Variance + 1,254)
      Endif
      FootNote = "[" + Footnote + "]"
      @BoxRow + InnerLength + 1,
       BoxCol + int(((InnerWidth + 4) - Len(FootNote))/2)
      ?? FootNote
      Style
   Endif

   ; Set object data for writing to window...
   Box_WindowCol = BoxCol + 2
   Box_WindowRow = BoxRow + WindowOffset + 1
   Box_WindowLength = InnerLength - WindowOffset
   Box_WindowWidth = InnerWidth
   Box_TitleColors = TitleColors
   Box_WindowColors = BodyColors
   Box_BorderColors = BorderColors
   Box_HighLightColors = HighLightColors

   ; Paint body...
   If BodyColors <> BorderColors Then
      PaintCanvas
         Attribute Box_WindowColors
         Box_WindowRow, Box_WindowCol,
         Box_WindowRow + Box_WindowLength - 1,
         Box_WindowCol + Box_WindowWidth - 1
   Endif
   Canvas On
Endproc
Writelib    LibrarySpec    Box.Display
Release     Procs          Box.Display

Proc Box.WriteRow(WriteRow,String)
   If WriteRow > Box_WindowLength Then
      Return
   Endif
   @ Box_WindowRow + WriteRow - 1, Box_WindowCol
   Style Attribute Box_WindowColors
   ?? Format("W" + StrVal(Box_WindowWidth),String)
   Style
EndProc
Writelib    LibrarySpec    Box.WriteRow
Release     Procs          Box.WriteRow

Proc Box.HighLightRow(HighLightRow)
   PaintCanvas
      Attribute Box_HighlightColors
      Box_WindowRow + HighLightRow - 1,
      Box_WindowCol,
      Box_WindowRow + HighLightRow - 1,
      Box_WindowCol + Box_WindowWidth - 1
EndProc
Writelib    LibrarySpec    Box.HighLightRow
Release     Procs          Box.HighLightRow

Proc Box.BodyColorRow(ColorRow)
   PaintCanvas
      Attribute Box_WindowColors
      Box_WindowRow + ColorRow - 1,
      Box_WindowCol,
      Box_WindowRow + ColorRow - 1,
      Box_WindowCol + Box_WindowWidth - 1
EndProc
Writelib    LibrarySpec    Box.BodyColorRow
Release     Procs          Box.BodyColorRow

Proc Box.MoreAboveArrow(LogicalValue)
   @Box_WindowRow,Box_WindowCol - 2
   Style Attribute Box_BorderColors
   If LogicalValue Then
      ?? ""
   Else
      ?? " "
   Endif
   Style
EndProc
Writelib    LibrarySpec    Box.MoreAboveArrow
Release     Procs          Box.MoreAboveArrow

Proc Box.MoreBelowArrow(LogicalValue)
   @Box_WindowRow + Box_WindowLength - 1,Box_WindowCol - 2
   Style Attribute Box_BorderColors
   If LogicalValue Then
      ?? ""
   Else
      ?? " "
   Endif
   Style
EndProc
Writelib    LibrarySpec    Box.MoreBelowArrow
Release     Procs          Box.MoreBelowArrow

Proc Box.ClearMessage()
   Box.BorderColorArea(0,0,0,0) ; to clear message
EndProc
Writelib    LibrarySpec    Box.ClearMessage
Release     Procs          Box.ClearMessage

Proc Box.BorderColorArea(TopLeftRow,TopLeftCol,
                       BottomRightRow,BottomRightCol)
   PaintCanvas
      Attribute Box_BorderColors
      Box_WindowRow + TopLeftRow - 1,
      Box_WindowCol + TopLeftCol - 1,
      Box_WindowRow + BottomRightRow - 1,
      Box_WindowCol + BottomRightCol - 1
EndProc
Writelib    LibrarySpec    Box.BorderColorArea
Release     Procs          Box.BorderColorArea

Proc Box.RegisterType(BoxType)
   Switch
      Case Boxtype = "MenuBox":
         Box.MenuBoxNumber    = Box.MenuBoxNumber + 1
         Box.MenuBoxIsAlive = True
         Return Box.MenuBoxNumber
      ; ...other kinds of popups...
   EndSwitch
EndProc
Writelib    LibrarySpec    Box.RegisterType
Release     Procs          Box.RegisterType

Proc Box.DeregisterType(BoxType)
   Switch
      Case Boxtype = "MenuBox":
         Box.MenuBoxNumber    = Box.MenuBoxNumber - 1
         Box.MenuBoxIsAlive = False
         If Box.MenuBoxNumber < 0 Then
            Debug ; Underflow box deregistration"
         Endif
         Return Box.MenuBoxNumber
      ; ...other kinds of popups...
   EndSwitch
EndProc
Writelib    LibrarySpec    Box.DeregisterType
Release     Procs          Box.DeregisterType
